package uk.ac.cardiff.booking.api.module;

import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import uk.ac.cardiff.booking.api.tutor.MissingTutorException;
import uk.ac.cardiff.booking.api.tutor.Tutor;
import uk.ac.cardiff.booking.api.tutor.TutorService;

@RestController
@RequestMapping("/modules")
public class ModuleController {
  
  @Autowired
  private ModuleRepository moduleRepo;

  @Autowired
  private ModuleService moduleService;

  @Autowired
  private TutorService tutorService;
  
  @PostMapping(value = "", 
    consumes=MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<Module> createModule(@RequestBody Module module) {
    Module savedModule = moduleRepo.save(module);
    return ResponseEntity.created(URI.create("/module/"+savedModule.getId())).body(savedModule);
  }

  @GetMapping("")
  public ResponseEntity<List<Module>> getModules() {
    List<Module> moduleList = new ArrayList<>();
    Iterator<Module> modules = moduleRepo.findAll().iterator();
    while (modules.hasNext()) {
      moduleList.add(modules.next());
    }
    return ResponseEntity.ok().body(moduleList);
  }

  @GetMapping("/{moduleCode}")
  public ResponseEntity<Module> getModule(@PathVariable("moduleCode") String moduleCode) {
    Module module = moduleRepo.findByModuleCode(moduleCode);
    return ResponseEntity.ok().body(module);
  }

  @PostMapping(value = "/{moduleCode}/tutors", 
    consumes=MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<Module> assignTutorToModule(@PathVariable("moduleCode") String moduleCode, @RequestBody Tutor tutor) {
    Module module = moduleRepo.findByModuleCode(moduleCode);
    try {
      Tutor tutorSearch = tutorService.findById(tutor.getId());
      module.getTutors().add(tutorSearch);
      moduleRepo.save(module);
    } catch(MissingTutorException ex) {
      return ResponseEntity.notFound().build();
    }
    return ResponseEntity.ok().body(module);
  }
}