package uk.ac.cardiff.booking.api.transcripts;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import lombok.extern.slf4j.Slf4j;

@RestController
@RequestMapping("/transcripts")
@Slf4j
public class TranscriptsController {

  private static final Logger LOG = LoggerFactory.getLogger(TranscriptsController.class);

  @Autowired
  private TranscriptsRepository repo;

  @PostMapping("/")
  public ResponseEntity<Transcript> createTranscript(@RequestBody Transcript transcript) {
    try {
      log.info(transcript.toString());
      Transcript savedTranscript = repo.save(transcript);
      return ResponseEntity.created(URI.create("/transcripts/" + savedTranscript.getId()))
            .body(savedTranscript);
    } catch(Exception ex) {
      LOG.error("Error", ex);
      return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
    }
  }

  @GetMapping("/{transcript_id}")
  public ResponseEntity<Transcript> getTranscript(@PathVariable("transcript_id") String transcriptId) {
    Transcript transcript = repo.findById(Integer.valueOf(transcriptId)).get();
    return ResponseEntity.ok().body(transcript);
  }

  @GetMapping("")
  public ResponseEntity<List<Transcript>> getTranscripts() {
    List<Transcript> transcripts = new ArrayList<>();
    repo.findAll().forEach(transcripts::add);
    return ResponseEntity.ok().body(transcripts);
  }

  @DeleteMapping("/{transcript_id}")
  public ResponseEntity<Transcript> deleteTranscript(@PathVariable("transcript_id") String transcriptId) {
    return ResponseEntity.noContent().build();
  }
}