package uk.ac.cardiff.booking.api.tutor;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/tutors")
public class AppointmentController {
  
  @Autowired
  private AppointmentService appointmentService;

  @GetMapping("/{tutor_id}/appointments")
  public ResponseEntity<List<AppointmentDto>> getAppointments(@PathVariable("tutor_id") Integer tutorId) {
    List<AppointmentDto> appointments = appointmentService.getAppointments(tutorId);
    return ResponseEntity.ok().body(appointments);
  }

  @PostMapping("/{tutor_id}/appointments")
  public ResponseEntity<AppointmentDto> createAppointment(@PathVariable("tutor_id") Integer tutorId,
      @RequestBody AppointmentDto appointment) {
    try {
      AppointmentDto appointmentDto = appointmentService.createAppointment(tutorId, appointment);
      return ResponseEntity.ok().body(appointmentDto);
    } catch(MissingTutorException ex) {
      return ResponseEntity.notFound().build();
    }
  }

  @DeleteMapping("/{tutor_id}/appointments/{appointmentId}")
  public ResponseEntity<Void> deleteAppointment(@PathVariable("tutor_id") Integer tutorId,
      @PathVariable("appointmentId") Integer appointmentId) {
    appointmentService.deleteAppointment(tutorId, appointmentId);
    return ResponseEntity.noContent().build();
  }
}