package uk.ac.cardiff.booking.api.module;

import java.util.HashSet;
import java.util.Set;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import uk.ac.cardiff.booking.api.tutor.Tutor;

@ExtendWith(MockitoExtension.class)
public class ModuleServiceUnitTest {
  
  @InjectMocks
  private ModuleService moduleService;

  @Mock
  private ModuleRepository moduleRepo;

  @Test
  public void willAddTutorToModule() {
    Tutor tutor = Tutor.builder().id(1).name("Dr Terry").build();
    Set<Tutor> tutors = new HashSet<>();
    tutors.add(tutor);
    Module module = Module.builder().tutors(tutors).build();
    Tutor additionalTutor = Tutor.builder().name("testName").build();

    Module testModule = moduleService.addTutorToModule(additionalTutor, module);

    Assertions.assertEquals(2, testModule.getTutors().size());
    Assertions.assertEquals(additionalTutor, testModule.getTutors().stream().filter(t -> t.getName().equals("testName")).findAny().get());
  }

  @Test
  public void willNotAddAtutorIfOneAlreadyExists() {
    Tutor tutor = Tutor.builder().id(1).name("Dr Terry").build();
    Set<Tutor> tutors = new HashSet<>();
    tutors.add(tutor);
    Module module = Module.builder().tutors(tutors).build();

    Module testModule = moduleService.addTutorToModule(tutor, module);

    Assertions.assertEquals(1, testModule.getTutors().size());
  }
}