package uk.ac.cardiff.booking.api.transcripts;

import static org.mockito.ArgumentMatchers.any;

import java.util.Optional;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;
import org.springframework.mock.web.MockMultipartFile;

@ExtendWith(MockitoExtension.class)
public class TranscriptsControllerUnitTest {

  @InjectMocks
  private TranscriptsController controller;

  @Mock
  private MockMultipartFile mockFile;

  @Mock
  private FileUploader mockFileUploader;

  @Mock
  private TranscriptsRepository repo;

  @Test
  public void testGetTranscriptReturns200() {
    Mockito.when(repo.findById(ArgumentMatchers.anyInt()))
      .thenReturn(Optional.of(new Transcript()));

    ResponseEntity<Transcript> transcriptResponse = controller.getTranscript("123");

    Assertions.assertEquals(200, transcriptResponse.getStatusCode().value());
  }

  @Test
  public void testCreateTranscriptReturns201withLocationHeader() {
    Mockito.when(repo.save(any(Transcript.class)))
      .thenReturn(Transcript.builder().id(1).build());
    ResponseEntity<Transcript> transcriptResponse = controller.createTranscript(new Transcript());

    Assertions.assertEquals(201, transcriptResponse.getStatusCode().value());
    Assertions.assertEquals("/transcripts/1", transcriptResponse.getHeaders().getLocation().toString());
  }

  @Test
  public void testDeleteTranscriptReturns204() {
    ResponseEntity<Transcript> transcriptResponse = controller.deleteTranscript("123");

    Assertions.assertEquals(204, transcriptResponse.getStatusCode().value());
  }
}
