package uk.ac.cardiff.booking.api.transcripts;

import java.util.List;

import javax.persistence.EntityManager;
import javax.sql.DataSource;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.junit.jupiter.SpringExtension;

@ExtendWith(SpringExtension.class)
@DataJpaTest
public class TranscriptsRepositoryUnitTest {
  
  @Autowired
  private DataSource dataSource;
  @Autowired
  private JdbcTemplate jdbcTemplate;
  @Autowired
  private EntityManager entityManager;
  @Autowired
  private TranscriptsRepository transcriptsRepository;

  /*
    Test inspired by https://reflectoring.io/spring-boot-data-jpa-test/
  */
  @Test
  void injectedComponentsAreNotNull(){
    Assertions.assertNotNull(dataSource);
    Assertions.assertNotNull(jdbcTemplate);
    Assertions.assertNotNull(entityManager);
    Assertions.assertNotNull(transcriptsRepository);
  }

  @Test
  public void willSaveTranscriptToDatabase() {
    Transcript transcript = Transcript.builder()
        .moduleCode("CMT-145")
        .studentNumber("12345678")
        .courseworkId(3)
        .build();
    Transcript savedTranscript = transcriptsRepository.save(transcript);

    Assertions.assertNotNull(savedTranscript.getId());
  }

  @Test
  public void willFindTranscriptByStudentNumber() {
    List<Transcript> transcripts = transcriptsRepository.findByStudentNumber("1234561");

    Assertions.assertEquals(2, transcripts.size());
  }
}