package uk.ac.cardiff.booking.api.tutor;

import java.time.LocalDateTime;
import java.util.List;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;

@ExtendWith(MockitoExtension.class)
public class AppointmentControllerUnitTest {
  
  @InjectMocks
  private AppointmentController controller;

  @Mock
  private AppointmentService appointmentService;

  @Test
  public void testGetListReturns200() {

    AppointmentDto appointment1 = AppointmentDto.builder().title("title 1").build();
    AppointmentDto appointment2 = AppointmentDto.builder().title("title 2").build();
    Mockito.when(appointmentService.getAppointments(1))
      .thenReturn(List.of(appointment1, appointment2));
    ResponseEntity<List<AppointmentDto>> appointmentResponse = controller.getAppointments(1);

    Assertions.assertEquals(200, appointmentResponse.getStatusCode().value());
    Assertions.assertEquals(appointment1, appointmentResponse.getBody().get(0));
  }

  @Test
  public void returns404IfNoTutorExistsWhenCreatingApointment() {
    Mockito.when(appointmentService.createAppointment(
      ArgumentMatchers.anyInt(), ArgumentMatchers.any(AppointmentDto.class))).thenThrow(new MissingTutorException());
    ResponseEntity<AppointmentDto> appointmentResponse = controller.createAppointment(1, AppointmentDto.builder().title("title").end(LocalDateTime.now()).start(LocalDateTime.now()).build());

    Assertions.assertEquals(404, appointmentResponse.getStatusCode().value());
  }
}