package uk.ac.cardiff.booking.api.tutor;

import static org.mockito.ArgumentMatchers.anyInt;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import uk.ac.cardiff.booking.api.module.Module;
import uk.ac.cardiff.booking.api.module.ModuleRepository;
import uk.ac.cardiff.booking.api.transcripts.Transcript;
import uk.ac.cardiff.booking.api.transcripts.TranscriptsRepository;

@ExtendWith(MockitoExtension.class)
public class AppointmentServiceUnitTest {
  
  @InjectMocks
  private AppointmentService appointmentService;

  @Mock
  private AppointmentRepository appointmentRepo;

  @Mock
  private TutorRepository tutorRepo;

  @Mock
  private TranscriptsRepository transcriptRepo;

  @Mock
  private ModuleRepository moduleRepository;

  @Test
  public void testMapToDto() {
    Appointment app1 = new Appointment();
    Appointment app2 = new Appointment();
    app1.setTitle("title 1");
    app1.setTranscript(new Transcript());
    app2.setTitle("title 2");
    app2.setTranscript(new Transcript());
    Mockito.when(appointmentRepo.findByTutorId(1))
      .thenReturn(List.of(app1, app2));

    List<AppointmentDto> appointments = appointmentService.getAppointments(1);

    Assertions.assertEquals(2, appointments.size());
    Assertions.assertEquals("title 1", appointments.get(0).getTitle());
  }

  @Test
  public void exceptionThrownIfNoTutor() {
    Mockito.when(tutorRepo.findById(ArgumentMatchers.anyInt()))
      .thenReturn(Optional.empty());
    Assertions.assertThrows(MissingTutorException.class, () -> {
      appointmentService.createAppointment(1, AppointmentDto.builder().build());
    });
  }

  @Test
  public void willReturnDtoWhenNewAppointmentCreated() {
    LocalDateTime start = LocalDateTime.of(2000, 01, 01, 01, 01);
    LocalDateTime end = LocalDateTime.of(2000, 01, 01, 01, 02);
    UUID authCode = UUID.fromString("83920475-fa74-1e98-dc1e-578e13ee7897");

    Mockito.when(tutorRepo.findById(1))
      .thenReturn(Optional.of(new Tutor()));

    AppointmentDto newDto = AppointmentDto.builder()
      .start(start)
      .end(end)
      .title("title")
      .moduleCode("module")
      .authCode(authCode)
      .transcriptId(1)
      .build();

    Appointment appt = new Appointment();
    appt.setId(1);
    Mockito.when(appointmentRepo.save(ArgumentMatchers.any(Appointment.class)))
      .thenReturn(appt);
    Mockito.when(moduleRepository.findByModuleCode(ArgumentMatchers.anyString()))
      .thenReturn(new Module());
    Mockito.when(transcriptRepo.findById(anyInt()))
      .thenReturn(Optional.of(Transcript.builder().id(1).build()));
    AppointmentDto dto = appointmentService.createAppointment(1, newDto);

    Assertions.assertEquals(AppointmentDto.builder()
      .start(start)
      .id(1)
      .end(end)
      .title("title")
      .transcriptId(1)
      .build(), dto);
  }
}