const express = require("express");
const cors = require("cors");
const path = require("path");
const bodyParser= require('body-parser')
const cookieParser = require("cookie-parser");
const cookieMiddleware = require("./session/cookie");
const calendarMiddleware = require("./session/calendarMiddleware");
const router = require("./routes/routes")
const moduleRoutes = require("./routes/module")
const calendarRoutes = require("./routes/calendar")
const studentRoutes = require("./routes/student")
const staffRoutes = require("./routes/staff")

let nunjucks = require("nunjucks");
let server = express();

server.use(cookieParser());
server.use(express.json());

server.use(express.urlencoded({ extended: false }));
server.use(cors());

server.use(bodyParser.urlencoded({extended: true}))

server.options("*", cors());

server.use("/", router)
server.use("/module", moduleRoutes)
server.use("/student", cookieMiddleware, studentRoutes)
server.use("/calendar", cookieMiddleware, calendarMiddleware, calendarRoutes)
server.use("/staff", staffRoutes)

const env = nunjucks.configure([
  "views",
], {
  autoescape: true,
  express: server,
});

server.set("views", path.join(__dirname, "views"));
server.set("view engine", "html");

server.use(express.static('public'))

server.listen(8080, function() {
  console.log("server listening");
});
