const express = require("express");
const fetch = require('node-fetch')
const router = express.Router();
const multer = require('multer');
var FormData = require('form-data');
const { Readable } = require('stream');
const session = require("../session/session");
const { response } = require("express");
const { start } = require("repl");
const { uuid } = require('uuidv4')

const API_HOST = process.env.API_HOST || "http://localhost";
const API_PORT = process.env.API_PORT || 8888;
const MINIO_HOST = process.env.MINIO_HOST || "localhost";
const MINIO_PORT = process.env.MINIO_PORT || 9000;
const MINIO_ACCESS_KEY = process.env.MINIO_ACCESS_KEY || "AKIAIOSFODNN7EXAMPLE";
const MINIO_SECRET_KEY = process.env.MINIO_SECRET_KEY || "wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY";

var Minio = require('minio');
const { json } = require("body-parser");

var minioClient = new Minio.Client({
    endPoint: MINIO_HOST,
    port: parseInt(MINIO_PORT),
    useSSL: false,
    accessKey: MINIO_ACCESS_KEY,
    secretKey: MINIO_SECRET_KEY
});

var storage = multer.memoryStorage({
  destination: function (req, file, cb) {
    cb(null, 'uploads')
  },
  filename: function (req, file, cb) {
    cb(null, file.fieldname + '-' + Date.now())
  }
})
 
var upload = multer({ storage: storage })

router.get("/new", (req,res) => {
  return res.render("student/new_student.html");
})

router.post("/new", async (req,res) => {
  console.log(req.body)
  const cookie = req.cookies["sessionId"];
  sessionToSave = {
    studentId: req.body.studentId,
    email: req.body.email,
    name: req.body.name,
    isTutor: false
  }
  session.upsertSession(cookie, JSON.stringify(sessionToSave))
  return res.redirect("/student/booking")
})

router.get("/booking", async (req,res) => {
  clientRes = await fetch(`${API_HOST}:${API_PORT}/modules`)
  responseBody = await clientRes.json()
  console.log(responseBody)
  return res.render("student/new_booking.html", {moduleCodes: responseBody})
})

router.get("/booking/existing", async (req,res) => {
  const cookie = req.cookies["sessionId"];
  console.log(cookie)
  sess = JSON.parse(await session.getSession(cookie))
  clientRes = await fetch(`${API_HOST}:${API_PORT}/bookings/student/${sess.studentId}`,
    {
      method: "GET",
      headers: { 
        'auth-code': cookie,
        'Accept': "application/json"
      }
    })
  apps = await clientRes.json()
  console.log(apps)

  displayAppointments = apps.map(app => {
    console.log(app)
    s = new Date(app.start)
    end = new Date(app.end)
    t = getTime(s) + " - " + getTime(end)
    return {
      moduleCode: app.moduleCode,
      tutor: app.tutor,
      date: getDate(s),
      time: t,
      tutorId: app.tutorId,
      id: app.id,
      transcriptName: app.transcriptName
    }
  })

  return res.render("student/existing_bookings.html", { bookings: displayAppointments})
})


// taken from 
// https://stackoverflow.com/questions/7357734/how-do-i-get-the-time-of-day-in-javascript-node-js
function getTime(date) {

  var hour = date.getHours();
  hour = (hour < 10 ? "0" : "") + hour;
  var min  = date.getMinutes();
  min = (min < 10 ? "0" : "") + min;
  return  hour + ":" + min;
}

function getDate(date) {

  var year = date.getFullYear();
  var month = date.getMonth() + 1;
  month = (month < 10 ? "0" : "") + month;
  var day  = date.getDate();
  day = (day < 10 ? "0" : "") + day;
  return  day + "/" + month + "/" + year;
}

router.get("/auth", async (req,res) => {
  return res.render("student/auth-code.html")
})

router.post("/auth", async (req,res) => {
  authCode = req.body.authCode
  res.cookie("sessionId", authCode, { maxAge: 900000000, httpOnly: true });
  return res.redirect("/student/booking/existing")
})

router.get("/booking/edit/:booking_id", async (req,res) => {
  code = req.cookies["sessionId"];
  clientRes = await fetch(`${API_HOST}:${API_PORT}/tutors/${req.query.tutorId}/appointments/${req.params.booking_id}`,
    {
      method: "DELETE",
      headers: { 
        'auth-code': code,
        'Accept': "application/json"
      }
    })
  return res.redirect("/student/booking/existing")
})

router.post("/booking", async (req,res) => {
  console.log(req.body)
  const cookie = req.cookies["sessionId"];
  console.log(cookie)
  sess = JSON.parse(await session.getSession(cookie))
  sess["moduleCode"] = req.body.module
  sess["moduleId"] = req.body.id

  await session.upsertSession(cookie, JSON.stringify(sess))
  return res.redirect("/student/tutor")
})

router.get("/tutor", async (req,res) => {
  const cookie = req.cookies["sessionId"];
  sess = JSON.parse(await session.getSession(cookie))
  clientRes = await fetch(`${API_HOST}:${API_PORT}/modules/${sess.moduleCode}`)
  responseBody = await clientRes.json()
  console.log(responseBody)
  return res.render("student/tutor_select.html", {tutors: responseBody.tutors})
})

router.post("/tutor", async (req,res) => {
  const cookie = req.cookies["sessionId"];
  sess = JSON.parse(await session.getSession(cookie))
  console.log(`tutor selected: ${req.body.tutorId}`)
  sess["tutorId"] = req.body.tutorId

  await session.upsertSession(cookie, JSON.stringify(sess))
  return res.redirect(`/student/transcripts`)
})

router.get("/transcripts", (req,res) => {
  return res.render("student/transcript_upload.html")
})

router.post("/transcripts", upload.single('file'), async (req,res) => {
  const cookie = req.cookies["sessionId"];
  sess = JSON.parse(await session.getSession(cookie))

  console.log(req.file);
  n = req.file.originalname.lastIndexOf(".")
  extension = req.file.originalname.substring(n + 1)
  filename = uuid()
  minioClient.putObject("transcripts", `${filename}.${extension}` , await req.file.buffer, async function(err, etag) {
    if (!err) {
      try {
        r = await fetch(`${API_HOST}:${API_PORT}/transcripts/`, {
          method: 'POST',
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify({
            studentNumber: sess.studentId,
            moduleCode: sess.moduleCode,
            filename: `${filename}.${extension}`,
            courseworkId: 0
          })
        })
        resp = await r.json()
        sess["transcriptId"] = resp.id
        await session.upsertSession(cookie, JSON.stringify(sess))
        return console.log(resp)
      } catch(ex) {
        console.log(ex)
      }
    }
    return console.log(err, etag) 
  })
  return res.redirect(`/calendar?tutorId=${sess["tutorId"]}`)
})

router.get("/transcripts/:transcript_id", (req, res) => {
  filename = req.params.transcript_id
  minioClient.presignedGetObject('transcripts', filename, 24*60*60, function(err, presignedUrl) {
    if (err) {
      return console.log(err);
    }
    console.log(presignedUrl);
    res.redirect(presignedUrl);
  })
})

module.exports = router