package booking.frontend.steps;

import static org.junit.jupiter.api.Assertions.assertTrue;

import org.openqa.selenium.WebDriver;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.annotation.DirtiesContext;

import booking.frontend.pages.HomePage;
import booking.frontend.pages.StaffPage;
import io.cucumber.java.en.And;
import io.cucumber.java.en.Given;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;

@DirtiesContext
public class StaffSteps {

  @Autowired
  private WebDriver webDriver;

  @Autowired
  private StaffPage staffPage;

  @Autowired
  private HomePage homePage;

  @Value("${frontend.username}")
  private String username;
  
  @Value("${frontend.password}")
  private String password;

  @Given("I log into the staff page")
  public void visit() {
    homePage.visit();
    homePage.clickLogin();
    staffPage.inputText("exampleInputEmail1", username)
             .inputText("exampleInputPassword1", password)
             .click("submitId");
  }

  @And("I add a tutor")
  public void addTutor() {
    staffPage.clickAddTutor();
    staffPage.inputText("inputName", "Dr bozak")
             .inputText("emailAddress", "bozak@cardiff.ac.uk")
             .click("addTutor");
  }

  @And("I add a module")
  public void addModule() {
    staffPage.clickAddModule();
    staffPage.inputText("moduleCode", "CMT-159")
             .inputText("moduleName", "Computing with Python")
             .click("addModule");
  }

  @When("I assign the module to the tutor")
  public void assignModule() throws InterruptedException {
    staffPage.clickAssignModule();
    staffPage.selectOption("moduleCode", "CMT-159")
             .selectOption("tutorId", "Dr bozak");
    staffPage.click("assignModule");
  }

  @Then("I am returned to the staff page")
  public void verifyStaff() {
    assertTrue(webDriver.getCurrentUrl().endsWith("staff"), "Unexpected page");
  }
}