package booking.frontend.steps;

import static org.junit.jupiter.api.Assertions.assertTrue;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.DirtiesContext;

import booking.Auth;
import booking.frontend.pages.HomePage;
import booking.frontend.pages.StudentPage;
import io.cucumber.java.en.And;
import io.cucumber.java.en.Given;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;

@DirtiesContext
public class StudentSteps {

  @Autowired
  private WebDriver webDriver;

  @Autowired
  private StudentPage studentPage;

  @Autowired
  private HomePage homePage;

  @Autowired
  private Auth data;

  @Given("I click the new booking tab")
  public void visit() {
    homePage.visit();
    homePage.clickBooking();
  }

  @And("I enter my student details")
  public void addDetails() {
    studentPage.inputText("emailAddress", "e.t@cardiff.ac.uk")
               .inputText("inputName", "Emma Terry")
               .inputText("inputId", "12345678")
               .click("submit");
  }

  @And("I select a module")
  public void addModule() {
    studentPage.selectOption("moduleId", "CMT-159")
               .click("submit");
  }

  @And("I select a tutor")
  public void addTutor() {
    studentPage.selectOption("tutorId", "Dr bozak")
             .click("submit");
  }

  @When("I upload a transcript")
  public void uploadTranscript() {
    studentPage.uploadFile();
  }

  @Then("I will be taken to the calendar page")
  public void verifyCalendar() {
    assertTrue(webDriver.getCurrentUrl().endsWith("calendar?tutorId=1"), "Unexpected page");
  }

  @When("I select an appointment")
  public void selectCalendar() throws InterruptedException {
    webDriver.findElements(By.xpath("//td[@data-time='09:00:00']")).get(1).click();
    webDriver.findElements(By.xpath("//button[@class='fc-submit-button fc-button fc-button-primary']")).get(0).click();
    Thread.sleep(1000);
    webDriver.switchTo().alert().accept();
  }

  @Then("I will be taken to the success page")
  public void verifySuccess() {
    assertTrue(webDriver.getCurrentUrl().endsWith("calendar/success"), "Unexpected page");
    String code = webDriver.findElement(By.className("text-muted")).getText();
    data.setAuthCode(code);
  }

  @Given("I click the view bookings page")
  public void viewBookings() {
    homePage.visit();
    homePage.clickExistingBooking();
  }

  @And("I enter the authcode")
  public void enterAuthCode() {
    studentPage.inputText("inputAuthCode", data.getAuthCode())
      .click("submit");
  }

  @Then("I will see the booking i previously made")
  public void viewBooking() {
    assertTrue(webDriver.getCurrentUrl().endsWith("student/booking/existing"), "Unexpected page");
  }
}